#include <jpype.h>
jbyte JPypeReference[] = {
    (jbyte)0xCA,(jbyte)0xFE,(jbyte)0xBA,(jbyte)0xBE,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x2E,(jbyte)0x00,(jbyte)0x21,(jbyte)0x01,(jbyte)0x00,(jbyte)0x18,(jbyte)0x6A,(jbyte)0x70,(jbyte)0x79,(jbyte)0x70,(jbyte)0x65,(jbyte)0x2F,(jbyte)0x72
    ,(jbyte)0x65,(jbyte)0x66,(jbyte)0x2F,(jbyte)0x4A,(jbyte)0x50,(jbyte)0x79,(jbyte)0x70,(jbyte)0x65,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x07,(jbyte)0x00,(jbyte)0x01
    ,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1E,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67,(jbyte)0x2F,(jbyte)0x72,(jbyte)0x65,(jbyte)0x66,(jbyte)0x2F,(jbyte)0x50,(jbyte)0x68,(jbyte)0x61
    ,(jbyte)0x6E,(jbyte)0x74,(jbyte)0x6F,(jbyte)0x6D,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x07,(jbyte)0x00,(jbyte)0x03,(jbyte)0x01,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x6D
    ,(jbyte)0x48,(jbyte)0x6F,(jbyte)0x73,(jbyte)0x74,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x01,(jbyte)0x4A,(jbyte)0x01,(jbyte)0x00,(jbyte)0x06
    ,(jbyte)0x3C,(jbyte)0x69,(jbyte)0x6E,(jbyte)0x69,(jbyte)0x74,(jbyte)0x3E,(jbyte)0x01,(jbyte)0x00,(jbyte)0x33,(jbyte)0x28,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67
    ,(jbyte)0x2F,(jbyte)0x4F,(jbyte)0x62,(jbyte)0x6A,(jbyte)0x65,(jbyte)0x63,(jbyte)0x74,(jbyte)0x3B,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67,(jbyte)0x2F,(jbyte)0x72
    ,(jbyte)0x65,(jbyte)0x66,(jbyte)0x2F,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x51,(jbyte)0x75,(jbyte)0x65,(jbyte)0x75,(jbyte)0x65,(jbyte)0x3B,(jbyte)0x29,(jbyte)0x56
    ,(jbyte)0x01,(jbyte)0x00,(jbyte)0x04,(jbyte)0x43,(jbyte)0x6F,(jbyte)0x64,(jbyte)0x65,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x08,(jbyte)0x0A,(jbyte)0x00,(jbyte)0x04,(jbyte)0x00,(jbyte)0x0A,(jbyte)0x01,(jbyte)0x00,(jbyte)0x0F
    ,(jbyte)0x4C,(jbyte)0x69,(jbyte)0x6E,(jbyte)0x65,(jbyte)0x4E,(jbyte)0x75,(jbyte)0x6D,(jbyte)0x62,(jbyte)0x65,(jbyte)0x72,(jbyte)0x54,(jbyte)0x61,(jbyte)0x62,(jbyte)0x6C,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x12,(jbyte)0x4C,(jbyte)0x6F
    ,(jbyte)0x63,(jbyte)0x61,(jbyte)0x6C,(jbyte)0x56,(jbyte)0x61,(jbyte)0x72,(jbyte)0x69,(jbyte)0x61,(jbyte)0x62,(jbyte)0x6C,(jbyte)0x65,(jbyte)0x54,(jbyte)0x61,(jbyte)0x62,(jbyte)0x6C,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x04,(jbyte)0x74
    ,(jbyte)0x68,(jbyte)0x69,(jbyte)0x73,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1A,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x70,(jbyte)0x79,(jbyte)0x70,(jbyte)0x65,(jbyte)0x2F,(jbyte)0x72,(jbyte)0x65,(jbyte)0x66,(jbyte)0x2F,(jbyte)0x4A,(jbyte)0x50,(jbyte)0x79
    ,(jbyte)0x70,(jbyte)0x65,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x3B,(jbyte)0x01,(jbyte)0x00,(jbyte)0x04,(jbyte)0x61,(jbyte)0x72,(jbyte)0x67,(jbyte)0x30,(jbyte)0x01
    ,(jbyte)0x00,(jbyte)0x12,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67,(jbyte)0x2F,(jbyte)0x4F,(jbyte)0x62,(jbyte)0x6A,(jbyte)0x65,(jbyte)0x63,(jbyte)0x74,(jbyte)0x3B
    ,(jbyte)0x01,(jbyte)0x00,(jbyte)0x04,(jbyte)0x61,(jbyte)0x72,(jbyte)0x67,(jbyte)0x31,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1E,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67
    ,(jbyte)0x2F,(jbyte)0x72,(jbyte)0x65,(jbyte)0x66,(jbyte)0x2F,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x51,(jbyte)0x75,(jbyte)0x65,(jbyte)0x75,(jbyte)0x65,(jbyte)0x3B
    ,(jbyte)0x01,(jbyte)0x00,(jbyte)0x10,(jbyte)0x67,(jbyte)0x65,(jbyte)0x74,(jbyte)0x48,(jbyte)0x6F,(jbyte)0x73,(jbyte)0x74,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x01
    ,(jbyte)0x00,(jbyte)0x03,(jbyte)0x28,(jbyte)0x29,(jbyte)0x4A,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x05,(jbyte)0x00,(jbyte)0x06,(jbyte)0x09,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x16,(jbyte)0x01,(jbyte)0x00,(jbyte)0x10,(jbyte)0x73,(jbyte)0x65
    ,(jbyte)0x74,(jbyte)0x48,(jbyte)0x6F,(jbyte)0x73,(jbyte)0x74,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x04,(jbyte)0x28,(jbyte)0x4A,(jbyte)0x29
    ,(jbyte)0x56,(jbyte)0x01,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x61,(jbyte)0x48,(jbyte)0x6F,(jbyte)0x73,(jbyte)0x74,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00
    ,(jbyte)0x08,(jbyte)0x68,(jbyte)0x61,(jbyte)0x73,(jbyte)0x68,(jbyte)0x43,(jbyte)0x6F,(jbyte)0x64,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x03,(jbyte)0x28,(jbyte)0x29,(jbyte)0x49,(jbyte)0x01,(jbyte)0x00,(jbyte)0x06,(jbyte)0x65,(jbyte)0x71
    ,(jbyte)0x75,(jbyte)0x61,(jbyte)0x6C,(jbyte)0x73,(jbyte)0x01,(jbyte)0x00,(jbyte)0x15,(jbyte)0x28,(jbyte)0x4C,(jbyte)0x6A,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x2F,(jbyte)0x6C,(jbyte)0x61,(jbyte)0x6E,(jbyte)0x67,(jbyte)0x2F,(jbyte)0x4F
    ,(jbyte)0x62,(jbyte)0x6A,(jbyte)0x65,(jbyte)0x63,(jbyte)0x74,(jbyte)0x3B,(jbyte)0x29,(jbyte)0x5A,(jbyte)0x01,(jbyte)0x00,(jbyte)0x0A,(jbyte)0x53,(jbyte)0x6F,(jbyte)0x75,(jbyte)0x72,(jbyte)0x63,(jbyte)0x65,(jbyte)0x46,(jbyte)0x69,(jbyte)0x6C
    ,(jbyte)0x65,(jbyte)0x01,(jbyte)0x00,(jbyte)0x13,(jbyte)0x4A,(jbyte)0x50,(jbyte)0x79,(jbyte)0x70,(jbyte)0x65,(jbyte)0x52,(jbyte)0x65,(jbyte)0x66,(jbyte)0x65,(jbyte)0x72,(jbyte)0x65,(jbyte)0x6E,(jbyte)0x63,(jbyte)0x65,(jbyte)0x2E,(jbyte)0x6A
    ,(jbyte)0x61,(jbyte)0x76,(jbyte)0x61,(jbyte)0x00,(jbyte)0x21,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x04,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x05,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x05,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x08,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x09,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x49,(jbyte)0x00,(jbyte)0x03,(jbyte)0x00
    ,(jbyte)0x03,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x07,(jbyte)0x2A,(jbyte)0x2B,(jbyte)0x2C,(jbyte)0xB7,(jbyte)0x00,(jbyte)0x0B,(jbyte)0xB1,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x0A,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x20,(jbyte)0x00,(jbyte)0x03
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x0F,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x10,(jbyte)0x00,(jbyte)0x11,(jbyte)0x00,(jbyte)0x01
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x12,(jbyte)0x00,(jbyte)0x13,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x14,(jbyte)0x00,(jbyte)0x15,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x09
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x2F,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x05,(jbyte)0x2A,(jbyte)0xB4,(jbyte)0x00,(jbyte)0x17,(jbyte)0xAD,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00
    ,(jbyte)0x02,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x11,(jbyte)0x00,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00
    ,(jbyte)0x01,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x05,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x0F,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x18,(jbyte)0x00,(jbyte)0x19,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00
    ,(jbyte)0x09,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x3E,(jbyte)0x00,(jbyte)0x03,(jbyte)0x00,(jbyte)0x03,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x2A,(jbyte)0x1F,(jbyte)0xB5,(jbyte)0x00,(jbyte)0x17,(jbyte)0xB1,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x0A,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x16,(jbyte)0x00,(jbyte)0x05,(jbyte)0x00,(jbyte)0x17,(jbyte)0x00
    ,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x16,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x0F,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00
    ,(jbyte)0x06,(jbyte)0x00,(jbyte)0x1A,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1B,(jbyte)0x00,(jbyte)0x1C,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x09,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00
    ,(jbyte)0x30,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x2A,(jbyte)0xB4,(jbyte)0x00,(jbyte)0x17,(jbyte)0x88,(jbyte)0xAC,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00
    ,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x1B,(jbyte)0x00,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x06,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x0F,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1D,(jbyte)0x00,(jbyte)0x1E,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x09,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x58,(jbyte)0x00,(jbyte)0x04,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x1C,(jbyte)0x2B,(jbyte)0xC1,(jbyte)0x00,(jbyte)0x02,(jbyte)0x9A,(jbyte)0x00,(jbyte)0x05,(jbyte)0x03,(jbyte)0xAC
    ,(jbyte)0x2B,(jbyte)0xC0,(jbyte)0x00,(jbyte)0x02,(jbyte)0xB4,(jbyte)0x00,(jbyte)0x17,(jbyte)0x2A,(jbyte)0xB4,(jbyte)0x00,(jbyte)0x17,(jbyte)0x94,(jbyte)0x9A,(jbyte)0x00,(jbyte)0x05,(jbyte)0x04,(jbyte)0xAC,(jbyte)0x03,(jbyte)0xAC,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x0C,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x03,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x20,(jbyte)0x00,(jbyte)0x07,(jbyte)0x00,(jbyte)0x22,(jbyte)0x00
    ,(jbyte)0x09,(jbyte)0x00,(jbyte)0x25,(jbyte)0x00,(jbyte)0x0D,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x16,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x1C,(jbyte)0x00,(jbyte)0x0E,(jbyte)0x00,(jbyte)0x0F,(jbyte)0x00
    ,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x1C,(jbyte)0x00,(jbyte)0x10,(jbyte)0x00,(jbyte)0x11,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x01,(jbyte)0x00,(jbyte)0x1F,(jbyte)0x00,(jbyte)0x00,(jbyte)0x00,(jbyte)0x02,(jbyte)0x00
    ,(jbyte)0x20
};
jsize getJPypeReferenceLength() { return 901; }
