//*****************************************************************************
//Copyright 2004-2008 Steve Menard
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//*****************************************************************************
package jpype.nio;

import java.nio.*;

class NioReceive 
{
  public static void receiveBuffer(ByteBuffer buffer)
  {
      System.out.println("Received a buffer of "+buffer.toString());
  }
  
  public static void allocSomeMemory()
  {
    byte[] b = new byte[10000];
  }

  public static void receiveBufferWithException(ByteBuffer buffer)
  {
      System.out.println("Received a buffer of "+buffer.toString());
      throw new RuntimeException("hello!");
  }
}